require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.graphics.Color"
import "android.net.Uri"
import "android.text.method.ScrollingMovementMethod"
import "android.speech.RecognizerIntent"
import "android.app.Activity"
import "android.graphics.Typeface"
import "android.view.Gravity"
import "com.androlua.Http"
local json = require "cjson"

-- API Configuration
local API_KEY = "AIzaSyDYeLxp7Jp5qSypbVBPy9v_XYmz7Sc1qfs"
local API_URL = "https://generativelanguage.googleapis.com/v1beta/models/gemini-1.5-flash-latest:generateContent?key=" .. API_KEY

activity.setTitle("AI Chatbot by Sujan")

-- References
local inputText
local chatContainer

-- Main Layout
mainLayout = {
    LinearLayout,
    orientation = "vertical",
    padding = "10dp",
    backgroundColor = "#FFFFFF",
    {
        TextView,
        text = "Welcome to AI Chatbot, powered by Gemini",
        textSize = "18sp",
        textColor = "#000000",
        gravity = "center",
        layout_width = "match_parent",
        Typeface = Typeface.DEFAULT_BOLD
    },
    {
        TextView,
        text = "Created by Sujan Rai at Tech Visionary",
        textSize = "14sp",
        textColor = "#555555",
        gravity = "center"
    },
    {
        Button,
        text = "Join Telegram",
        layout_marginTop = "10dp",
        onClick = function()
            local intent = Intent(Intent.ACTION_VIEW, Uri.parse("https://t.me/techvisionarynepal"))
            activity.startActivity(intent)
        end
    },
    {
        ScrollView,
        layout_weight = 1,
        {
            LinearLayout,
            id = "chatContainer",
            orientation = "vertical"
        }
    },
    {
        LinearLayout,
        orientation = "horizontal",
        gravity = "center_vertical",
        {
            TextView,
            text = "You:",
            layout_marginRight = "5dp",
            textSize = "16sp"
        },
        {
            EditText,
            id = "inputText",
            layout_weight = 1,
            hint = "Type your message...",
            textSize = "16sp"
        },
        {
            Button,
            text = "Send",
            onClick = function()
                local prompt = inputText.getText().toString()
                if prompt ~= "" then
                    sendToGemini(prompt)
                    inputText.setText("")
                end
            end
        },
        {
            Button,
            text = "Mic",
            onClick = function()
                local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
                intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
                intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Speak now...")
                activity.startActivityForResult(intent, 1000)
            end
        }
    }
}

activity.setContentView(loadlayout(mainLayout))

inputText = _G["inputText"]
chatContainer = _G["chatContainer"]

-- Add message to chat view
function addChat(text)
    local msg = TextView(activity)
    msg.setText(text)
    msg.setTextColor(0xFF000000)
    msg.setTextSize(16)
    msg.setPadding(5, 5, 5, 5)
    chatContainer.addView(msg)
end

-- Send to Gemini API
function sendToGemini(prompt)
    addChat("You said: " .. prompt)

    local requestBody = {
        contents = {
            {
                parts = {
                    { text = prompt }
                }
            }
        }
    }

    local headers = {
        ["Content-Type"] = "application/json"
    }

    -- Use this signature: post(url, json_string, headers_table, callback)
    Http.post(API_URL, json.encode(requestBody), headers, function(code, body)
        if code == 200 then
            local ok, res = pcall(json.decode, body)
            if ok and res and res.candidates and res.candidates[1] then
                local reply = res.candidates[1].content.parts[1].text or "No response"
                addChat("AI said: " .. reply)
            else
                addChat("AI said: [Invalid response format]")
            end
        else
            addChat("AI said: [HTTP Error " .. tostring(code) .. "]")
        end
    end)
end

-- Speech recognition result
function onActivityResult(requestCode, resultCode, data)
    if requestCode == 1000 and resultCode == Activity.RESULT_OK then
        local result = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
        if result and result.size() > 0 then
            inputText.setText(result.get(0))
        end
    end
end